﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Reactive.Linq;
using System.Threading.Tasks;
using Dapper;
using ExchangeService.Client.Models;
using ExchangeService.GrpcServer;
using Grpc.Core;
using Grpc.Net.Client;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Serilog;

namespace ExchangeService.Client.Controllers
{
    [Route("api/[controller]")]   
    [ApiController]

    public class DicomTransferFileController : ControllerBase
    {
        #region Properties
        private readonly ILogger<DicomTransferFileController> _logger;
        private readonly IConfiguration _configuration;
        private List<ImageDetails> _imageDetailsList;
        private List<DicomFilesTransferModel> _uploadedFiles;
        #endregion


        #region Constractor
        public DicomTransferFileController(ILogger<DicomTransferFileController> logger, IConfiguration config)
        {
            _logger = logger;
            _configuration = config;
        }
        #endregion

        //[Authorize]
        #region Methods
        [HttpPost]
        public async Task Post([FromForm]Models.FilesPackage queryFilters)
        {

            Log.Information("DicomTransferFile Started ...!");
            SendCallback sc = new SendCallback();
            string connectionString = _configuration.GetConnectionString("DefaultConnection");
            string view = _configuration["Views:ImageDetails"];
            _imageDetailsList = new List<ImageDetails>();
            _uploadedFiles = new List<DicomFilesTransferModel>();

            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                var query = connection.Query<ImageDetails>
                ($"SELECT *  from {view} where [StudyInstanceUID]='{queryFilters.StudyInstanceUID}'").ToList();
                _imageDetailsList.AddRange(query);
            }

            if (_imageDetailsList!.Count > 0 && _imageDetailsList.Select(f => f.File).Count() > 0)
            {
                _imageDetailsList.ToObservable().Subscribe(f =>
            {
                _uploadedFiles.Add(new DicomFilesTransferModel
                {
                    UploadedFile = f.File
                });
            });

                var channel = GrpcChannel.ForAddress("https://localhost:5001");
                var client =new DicomOps.DicomOpsClient(channel);
                _uploadedFiles.ToObservable().Subscribe(uploadedFile =>
                {
                    var call = client.TransferDicomFiles(uploadedFile);
                    while (call.ResponseStream.MoveNext().GetAwaiter().GetResult())
                    {
                        var currentCustomer = call.ResponseStream.Current;

                        Log.Information(currentCustomer.Message);
                        sc.OnOperationSuccessed();
                    }
                }
                );
            }
            else
            {
                Log.Error("There is no file to send");
                sc.OnOperationFailed();
            }
            Log.Information("operation Finished!");
            sc.OnSendingFinished();
        }
        #endregion

    }

    public class SendCallback : ISendCallback
    {
        #region Evnets
        public event EventHandler SendingFinished;
        public event EventHandler OperationFailed;
        public event EventHandler OperationSuccessed;
        #endregion

        #region Callback methods
        public void OnSendingFinished()
        {
            Console.WriteLine("OnSendingFinished at {0}", DateTime.Now);
            SendingFinished?.Invoke(this, null);
        }


        public void OnOperationFailed()
        {
            Console.WriteLine("OnOperationFailed at {0}", DateTime.Now);
            if (OperationFailed != null)
                OperationFailed(this, null);
        }

        public void OnOperationSuccessed()
        {
            Console.WriteLine("OnVerified at {0}", DateTime.Now);
            if (OperationSuccessed != null)
                OperationSuccessed(this, null);
        }




        #endregion
    }

    public interface ISendCallback
    {
        void OnSendingFinished();
        void OnOperationFailed();        
        void OnOperationSuccessed();
    }
}
