﻿
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Serilog;
using Serilog.Core;
using Serilog.Events;
using System;

using System.Collections.Generic;
using System.Text;

namespace ExchangeService.Client.SerilogConfig
{

    public class SeriLogConfig
    {
        static Microsoft.Extensions.Logging.ILogger _log;
        //public LineupLogger(Microsoft.Extensions.Logging.ILogger logger)
        //{
        //    if (logger != null)
        //    {
        //        _log = logger;
        //    }
        //}
        public static Microsoft.Extensions.Logging.ILogger GetLogger()
        {
            return _log;
        }
        public static (bool Result, string Log) LoggerConfiguration(LogLevel logLevel)
        {
            LogEventLevel eventLevel = (LogEventLevel)logLevel;
            return LoggerConfiguration(eventLevel);
        }
        public static (bool Result, string Log) LoggerConfiguration(LogEventLevel eventLevel)
        {
            try
            {
                var levelSwitch = new LoggingLevelSwitch();
                levelSwitch.MinimumLevel = eventLevel;
                #region Logger Config
                var conf = new LoggerConfiguration()
                    .MinimumLevel.ControlledBy(levelSwitch)
                    .Enrich.WithCaller()
                    .WriteTo.Console(outputTemplate: @"time=""{Timestamp:HH:mm:ss.fff}"" app=""{App}"" action=""{Action}"" level=""{Level:u3}"" msg=""{Message}""{NewLine:l}{Exception:l}")
                    .WriteTo.File(@"d:\logs\msrcore-client.txt", outputTemplate: @"time=""{Timestamp:HH:mm:ss.fff}"" app=""{App}"" action=""{Action}"" level=""{Level:u3}"" msg=""{Message}""{NewLine:l}{Exception:l}",
                    fileSizeLimitBytes: 10485760,
                    rollingInterval: RollingInterval.Day, shared: true,
                    rollOnFileSizeLimit: true);

                Log.Logger = conf.CreateLogger(); ;

                var serviceCollection = new ServiceCollection();
                serviceCollection.AddLogging(configure => configure.AddSerilog())
                        .AddTransient<Program>();
                var serviceProvider = serviceCollection.BuildServiceProvider();

                var logger = serviceProvider.GetService<ILogger<Program>>();
                _log = logger;
                return (true, "");
                #endregion
            }
            catch (Exception ex)
            {
                return (false, ex.ToString());
            }

        }


    }
}
