﻿using Serilog;
using Serilog.Configuration;
using Serilog.Core;
using Serilog.Events;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace ExchangeService.GrpcServer.SerilogConfig
{
    public static class SerilogExtention
    {

        public static LoggerConfiguration WithCaller(this LoggerEnrichmentConfiguration enrichmentConfiguration)
        {
            return enrichmentConfiguration.With<CallerEnricher>();
        }

    }
    class CallerEnricher : ILogEventEnricher
    {
        public void Enrich(LogEvent logEvent, ILogEventPropertyFactory propertyFactory)
        {
            var skip = 3;
            while (true)
            {
                var stack = new StackFrame(skip);
                if (!stack.HasMethod())
                {
                    logEvent.AddPropertyIfAbsent(new LogEventProperty("App", new ScalarValue("<unknown application>")));
                    logEvent.AddPropertyIfAbsent(new LogEventProperty("Action", new ScalarValue("<unknown method>")));
                    return;
                }

                var method = stack.GetMethod();
                if (method.DeclaringType.Assembly != typeof(Log).Assembly)
                {
                    // var caller = $"{method.DeclaringType.FullName}.{method.Name}({string.Join(", ", method.GetParameters().Select(pi => pi.ParameterType.FullName))})";
                    string app = $"{method.DeclaringType.FullName}";
                    string action = $"{method.Name}";
                    logEvent.AddPropertyIfAbsent(new LogEventProperty("App", new ScalarValue(app)));
                    logEvent.AddPropertyIfAbsent(new LogEventProperty("Action", new ScalarValue(action)));
                    return;
                }

                skip++;
            }
        }
    }
}
