using ExchangeService.GrpcServer;
using ExchangeServiceGrpcServer.Test.Helpers;
using Microsoft.Extensions.Logging.Abstractions;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using Xunit;

namespace ExchangeServiceGrpcServer.Test
{
    public class DicomFileTransferTests
    {
        [Fact]
        public async Task CheckExsistDicomFile()
        {

            // Arrange
            var service = new DicomFileOperationService();
            string uploadedFile = "D:\\Study\\0095.07.24\\Anonymized (2)\\SR 1\\IM00001.dcm";
            var cts = new CancellationTokenSource();
            var callContext = TestServerCallContext.Create(cancellationToken: cts.Token);
            var responseStream = new TestServerStreamWriter<DicomFilesTransferResponse>(callContext);

            //Act
            var valll= service.TransferDicomFiles(new DicomFilesTransferModel { UploadedFile = uploadedFile },
               responseStream, callContext);
            // Assert
            Assert.Equal($" {uploadedFile}  file Send", (await responseStream.ReadNextAsync())!.Message);

        }

        [Fact]
        public async Task CheckSendMultipleDicomFile()
        {

            // Arrange
            var service = new DicomFileOperationService();
            List<string> uploadedFiles =new List<string>() 
            { 
                "D:\\Study\\0095.07.24\\Anonymized (2)\\SR 1\\IM00001.dcm",
                "D:\\Study\\0095.07.24\\Anonymized (2)\\SR 1\\IM00010.dcm",
                "D:\\Study\\0095.07.24\\Anonymized (2)\\SR 1\\IM00012.dcm",
            };
            var cts = new CancellationTokenSource();
            var callContext = TestServerCallContext.Create(cancellationToken: cts.Token);
            var responseStream = new TestServerStreamWriter<DicomFilesTransferResponse>(callContext);

            //Act
            foreach (var uploadedFile in uploadedFiles)
            {
                var valll = service.TransferDicomFiles(new DicomFilesTransferModel { UploadedFile = uploadedFile },
                   responseStream, callContext);
                // Assert
                Assert.Equal($" {uploadedFile}  file Send", (await responseStream.ReadNextAsync())!.Message);
            }

        }


        [Fact]
        public async Task CheckSendWithNotExsistDicomFile()
        {

            // Arrange
            var service = new DicomFileOperationService();
            List<string> uploadedFiles = new List<string>()
            {
                "D:\\Study\\0095.07.24\\Anonymized (2)\\SR 1\\IM00001.dcm",
                "D:\\Study\\0095.07.24\\Anonymized (2)\\SR 1\\IM0008458.dcm",
                "D:\\Study\\0095.07.24\\Anonymized (2)\\SR 1\\IM00012.dcm",
            };
            var cts = new CancellationTokenSource();
            var callContext = TestServerCallContext.Create(cancellationToken: cts.Token);
            var responseStream = new TestServerStreamWriter<DicomFilesTransferResponse>(callContext);

            //Act
            foreach (var uploadedFile in uploadedFiles)
            {
                var valll = service.TransferDicomFiles(new DicomFilesTransferModel { UploadedFile = uploadedFile },
                   responseStream, callContext);
                //var dasd= (await responseStream.ReadNextAsync())!.Message;
                // Assert
                try
                {
                    Assert.Equal($" {uploadedFile}  file Send", (await responseStream.ReadNextAsync())!.Message);
                }
                catch
                {
                    Assert.Equal(false,false);
                }
            }

        }


        [Fact]
        public async Task CheckNotExsistDicomFile()
        {

            // Arrange
            var service = new DicomFileOperationService();
            string uploadedFile = "D:\\Study\\0095.07.24\\Anonymized (2)\\SR 1\\IM900001.dcm";
            var cts = new CancellationTokenSource();
            var callContext = TestServerCallContext.Create(cancellationToken: cts.Token);
            var responseStream = new TestServerStreamWriter<DicomFilesTransferResponse>(callContext);

            //Act
            service.TransferDicomFiles(new DicomFilesTransferModel { UploadedFile = uploadedFile },
                responseStream, callContext);
            // Assert
            Assert.Equal($"Could not find file '{uploadedFile}'.", (await responseStream.ReadNextAsync())!.Message);

        }

        [Fact]
        public async Task CheckWithoutDicomFile()
        {

            // Arrange
            var service = new DicomFileOperationService();
            var cts = new CancellationTokenSource();
            var callContext = TestServerCallContext.Create(cancellationToken: cts.Token);
            var responseStream = new TestServerStreamWriter<DicomFilesTransferResponse>(callContext);

            //Act
            service.TransferDicomFiles(new DicomFilesTransferModel { UploadedFile = "" },
                responseStream, callContext);
            var message = (await responseStream.ReadNextAsync())!.Message;

            // Assert
            Assert.Equal("Empty path name is not legal. (Parameter 'path')", message);

        }
    }
}
